IF EXISTS (SELECT * FROM sys.objects WHERE [type] IN (N'P', N'PC') AND OBJECT_ID = OBJECT_ID(N'[Common].[UpdateContractorFromXML]'))
DROP PROCEDURE [Common].[UpdateContractorFromXML];
GO

-- Procedura umoliwiajca zapisanie danych kontrahenta z postaci XML
CREATE PROCEDURE [Common].[UpdateContractorFromXML] (@contractorData XML, @moduleCode nvarchar(255), @contextInfo varchar(128), @contextInfoP varchar(128))
AS
BEGIN
	SET NOCOUNT ON; -- Stops the message that shows the count of the number of rows affected by a Transact-SQL statement or stored procedure from being returned as part of the result set.
	SET XACT_ABORT ON; -- Specifies whether SQL Server automatically rolls back the current transaction when a Transact-SQL statement raises a run-time error.

	--Ustawienie CONTEXT_INFO z danymi potrzebnymi trigerom do zapisu historii zmian w tabelach z sufiksem 'Rep'
	declare @ContextInfoBinary as varbinary(128)
	SET @ContextInfoBinary = convert(varbinary(128), @contextInfo)
	SET CONTEXT_INFO @ContextInfoBinary;
	declare @ContextInfoBinaryP as varbinary(128)
	SET @ContextInfoBinaryP = convert(varbinary(128), @contextInfoP)

	BEGIN TRY
		BEGIN TRANSACTION;
		
		-- tabela tymczasowa na dane kontrahenta z xml
		CREATE TABLE [#tblContractor]
		(
			[Id] INT,
			[Guid] UNIQUEIDENTIFIER,
			[Name] NVARCHAR(150),
			[Shortcut] NVARCHAR(50),
			[Nip] NVARCHAR(20),
			[NipUE] NVARCHAR(20),
			[Pesel] NVARCHAR(15),
			[Regon] NVARCHAR(20),
			[LinkedUnit] BIT,
			[PersonKind] TINYINT,
			[VATPayerActive] BIT,
			[VIES] BIT,
				-- exAttribute
			[RequestMppTransfers] NVARCHAR(10), -- splitPayment
			[PaymentFormId] INT, -- formaPlatnosci
			[PayRegistryId] INT, -- rejestr
			[SaleKindId] INT, -- rodzajCeny
			[PriceTypeId] INT, -- typCeny
			[AccountantParameter] NVARCHAR(20), -- parametrFK
			[IdFk] INT, -- idFK
			[PositionFk] INT, -- pozycja FK
			-- additional data
			[CatalogId] NVARCHAR(100),
			[ContractorKindId] INT,
			[MarkerId] INT,
			[Status] BIT,
			-- sale conditions
			[IsLimitActive] BIT,
			[LimitCurrency] NVARCHAR(20),
			[LimitValue] FLOAT,
			[PriceNegotiation] BIT,
			-- main person
			[FirstName] NVARCHAR(50),
			[Surname] NVARCHAR(50)
		)
		-- tabela tymczasowa na dane kontaktowe kontrahenta z xml
		CREATE TABLE [#tblContact]
		(
			[Id] INT,
			[Guid] UNIQUEIDENTIFIER,
			[FirstName] NVARCHAR(50),
			[Surname] NVARCHAR(50),
			[Email] NVARCHAR(100),
			[Facebook] NVARCHAR(100),
			[Fax] NVARCHAR(50),
			[Telephone1] NVARCHAR(50),
			[Telephone2] NVARCHAR(50),
			[Telex] NVARCHAR(50),
			[Website] NVARCHAR(100)
		)
		-- tabela tymczasowa na dane adresu domylnego kontrahenta z xml
		CREATE TABLE [#tblMainAddress]
		(
			[Id] INT,
			[Guid] UNIQUEIDENTIFIER,
			[Name] NVARCHAR(100),
			[Post] NVARCHAR(50),
			[PostalCode] NVARCHAR(10),
			[Place] NVARCHAR(50),
			[Street] NVARCHAR(50),
			[HouseNo] NVARCHAR(15),
			[ApartmentNo] NVARCHAR(15),
			[Province] NVARCHAR(50),
			[Country] NVARCHAR(50)
		)
		-- tabela tymczasowa na dane adresu korespondencyjnego kontrahenta z xml
		CREATE TABLE [#tblCorespondenceAddress]
		(
			[Id] INT,
			[Guid] UNIQUEIDENTIFIER,
			[Name] NVARCHAR(100),
			[Post] NVARCHAR(50),
			[PostalCode] NVARCHAR(10),
			[Place] NVARCHAR(50),
			[Street] NVARCHAR(50),
			[HouseNo] NVARCHAR(15),
			[ApartmentNo] NVARCHAR(15),
			[Province] NVARCHAR(50),
			[Country] NVARCHAR(50)
		)
		-- tabela tymczasowa na dane handlowe kontrahenta z xml
		CREATE TABLE [#tblTradeData]
		(
			[DiscountPercent] FLOAT,
			[Receivables] FLOAT,
			[Liabilities] FLOAT
		)
		-- tabela tymczasowa na pola z rachunkami bankowymi
		CREATE TABLE [#tblBankAccounts]
		(
			[Id] INT,
			[Guid] UNIQUEIDENTIFIER,
			[AccountNo] NVARCHAR(50),
			[AccountName] NVARCHAR(100),
			[IsDefault] BIT,
			[IsActive] BIT,
			[WhiteList] BIT,
			[WhiteListDate] DATETIME2,
			[SWIFT_BIC] VARCHAR(11),
			[BankName] NVARCHAR(100),
			-- extended attributes
			[IsElectronic] BIT,
			[IsVatAccount] BIT
		)
		-- tabela tymczasowa na pola z rachunkami bankowymi
		CREATE TABLE [#tblCustomFields]
		(
			[FieldName] NVARCHAR(100),
			[Value] NVARCHAR(MAX),
			[ColumnName] NVARCHAR(128),
			[Type] NVARCHAR(250)
		)
		
		-- tabela tymczasowa na ElementExAttributes
		CREATE TABLE [#ExtAttributes]
		(
			[ElementId] INT,
			[Name] NVARCHAR(50),
			[StringValue] NVARCHAR(1024),
			[OriginalCLRType] NVARCHAR(400),
			[ElementExAttrId] INT IDENTITY(1,1),
			[Guid] UNIQUEIDENTIFIER,
			[Title] NVARCHAR(200),
			[AccessFlag] BIT,
			[InheritedFlag] BIT
		)
		
		DECLARE @extAttElementId int
		DECLARE @extAttTitle nvarchar (200)
		DECLARE @extAttActive bit
		DECLARE @extAttAccountNo nvarchar (40)
		DECLARE @extAttDescription nvarchar (4000)
		DECLARE @extAttShortcut nvarchar (100)
		DECLARE @extAttPosition int
		DECLARE @extAttExtAttributes nvarchar(max)
		DECLARE @extAttExtObjVer int
		DECLARE @extAttExtElementKindGuid uniqueidentifier
		DECLARE @extAttExtWriteAction int
		DECLARE @extAttExtOtherThenOriginal smallint

		-- przygotowanie xml
		DECLARE @handle INT, @prepareXmlStatus INT
		EXEC @prepareXmlStatus = sp_xml_preparedocument @handle OUTPUT, @contractorData

		-- deserializacja danych kontaktowych z xml
		INSERT INTO [#tblContact]
		SELECT 
			ISNULL([Id], 0), CASE WHEN [Guid] IS NULL OR [Guid] = '' OR [Guid] = '00000000-0000-0000-0000-000000000000' THEN NEWID() ELSE [GUID] END,
			[FirstName], [SurName], [Email], [Facebook], [Fax], [Telephone1], [Telephone2], [Telex], [Website]
		FROM 
			OPENXML(@handle, '/Contractor/ContactPerson')
			WITH
			(
				[Id] INT 'Id',
				[Guid] NVARCHAR(36) 'Guid',
				[FirstName] NVARCHAR(50) 'FirstName',
				[Surname] NVARCHAR(50) 'SurName',
				[Email] NVARCHAR(100) 'Email',
				[Facebook] NVARCHAR(100) 'Facebook',
				[Fax] NVARCHAR(50) 'Fax',
				[Telephone1] NVARCHAR(50) 'Telephone1',
				[Telephone2] NVARCHAR(50) 'Telephone2',
				[Telex] NVARCHAR(50) 'Tlf',
				[Website] NVARCHAR(100) 'Website'
			)
		-- wyczyszczenie pustych elementw
		DELETE FROM [#tblContact]
		WHERE [FirstName] = '' AND [Surname] = '' AND [Email] = '' AND [Facebook] = '' AND [Fax] = '' AND [Telephone1] = '' AND [Telephone2] = '' AND [Telex] = '' AND [Website] = ''
		--select * from #tblContact

		-- deserializacja danych adresu domylnego z xml
		INSERT INTO [#tblMainAddress]
		SELECT 
			ISNULL([Id], 0),
			CASE WHEN [Guid] IS NULL OR [Guid] = '' THEN '00000000-0000-0000-0000-000000000000' ELSE [GUID] END,
			[Name], [Post], [PostalCode], [Place], [Street], [HouseNo], [ApartmentNo], [Province], [Country]
		FROM
			OPENXML(@handle, '/Contractor/MainAddress')
			WITH
			(
				[Id] INT 'Id',
				[Guid] NVARCHAR(36) 'Guid',
				[Name] NVARCHAR(100) 'Name',
				[Post] NVARCHAR(50) 'Post',
				[PostalCode] NVARCHAR(10) 'PostalCode',
				[Place] NVARCHAR(50) 'Place',
				[Street] NVARCHAR(50) 'Street',
				[HouseNo] NVARCHAR(15) 'HouseNo',
				[ApartmentNo] NVARCHAR(15) 'ApartmentNo',
				[Province] NVARCHAR(50) 'Province',
				[Country] NVARCHAR(50) 'Country'
			)
		-- wyczyszczenie pustych elementw
		DELETE FROM [#tblMainAddress]
		WHERE [Name] = '' AND [Post] = '' AND [PostalCode] = '' AND [Place] = '' AND [Street] = '' AND [HouseNo] = '' AND [ApartmentNo] = '' AND [Province] = '' AND [Country] = ''
		--select * from [#tblMainAddress]
		
		-- deserializacja danych adresu korespondencyjnego z xml
		INSERT INTO [#tblCorespondenceAddress]
		SELECT 
			ISNULL([Id], 0), CASE WHEN [Guid] IS NULL OR [Guid] = '' THEN '00000000-0000-0000-0000-000000000000' ELSE [GUID] END,
			[Name], [Post], [PostalCode], [Place], [Street], [HouseNo], [ApartmentNo], [Province], [Country]
		FROM 
			OPENXML(@handle, '/Contractor/CorespondenceAddress')
			WITH
			(
				[Id] INT 'Id',
				[Guid] NVARCHAR(36) 'Guid',
				[Name] NVARCHAR(100) 'Name',
				[Post] NVARCHAR(50) 'Post',
				[PostalCode] NVARCHAR(10) 'PostalCode',
				[Place] NVARCHAR(50) 'Place',
				[Street] NVARCHAR(50) 'Street',
				[HouseNo] NVARCHAR(15) 'HouseNo',
				[ApartmentNo] NVARCHAR(15) 'ApartmentNo',
				[Province] NVARCHAR(50) 'Province',
				[Country] NVARCHAR(50) 'Country'
			)
		-- wyczyszczenie pustych elementw
		DELETE FROM [#tblCorespondenceAddress]
		WHERE [Name] = '' AND [Post] = '' AND [PostalCode] = '' AND [Place] = '' AND [Street] = '' AND [HouseNo] = '' AND [ApartmentNo] = '' AND [Province] = '' AND [Country] = ''
		--select * from [#tblCorespondenceAddress]
		
		-- deserializacja danych handlowych z xml
		INSERT INTO [#tblTradeData]
		SELECT 
			[DiscountPercent], [Receivables], [Liabilities]
		FROM 
			OPENXML(@handle, '/Contractor')
			WITH
			(
				[DiscountPercent] FLOAT 'DiscountPercent',
				[Receivables] FLOAT 'Receivables',
				[Liabilities] FLOAT 'Liabilities'
			)
		DELETE FROM [#tblTradeData]
		WHERE [DiscountPercent] = 0 AND [Receivables] = 0 AND [Liabilities] = 0
		--select * from [#tblTradeData]
		
		-- deserializacja danych rachunkw bankowych z xml
		INSERT INTO [#tblBankAccounts]
		SELECT 
			[Id],CASE WHEN [Guid] IS NULL OR [Guid] = '' OR [Guid] = '00000000-0000-0000-0000-000000000000' THEN NEWID() ELSE [GUID] END,
			[AccountNo], [AccountName], [IsDefault], [IsActive],
			CASE WHEN [WhiteList] IS NULL OR [WhiteList] = '' THEN NULL ELSE (CASE WHEN [WhiteList] = 'true' THEN 1 ELSE 0 END) END [WhiteList],
			CASE WHEN [WhiteListDate] IS NULL OR [WhiteListDate] = '' THEN NULL ELSE cast([WhiteListDate] as DATETIME2) END [WhiteListDate],
			[SWIFT_BIC], [BankName],
			-- extended attributes
			[IsElectronic], [IsVatAccount]
		FROM 
			OPENXML(@handle, '/Contractor/BankAccounts/BankAccount')
			WITH
			(
				[Id] INT 'Id',
				[Guid] NVARCHAR(36) 'Guid',
				[AccountNo] NVARCHAR(50) 'AccountNo',
				[AccountName] NVARCHAR(100) 'AccountName',
				[IsDefault] BIT 'IsDefault',
				[IsActive] BIT 'IsActive',
				[WhiteList] NVARCHAR(5) 'IsInTaxPayerRegistry',
				[WhiteListDate] NVARCHAR(40) 'VerificationDate',
				[SWIFT_BIC] VARCHAR(11) 'SwiftBicNumber',
				[BankName] NVARCHAR(100) 'BankName',
				-- extended attributes
				[IsElectronic] BIT 'IsElectronic',
				[IsVatAccount] BIT 'IsVatAccount'
			)
			
		DELETE FROM [#tblBankAccounts]
		WHERE ([AccountNo] IS NULL OR [AccountNo] = '') AND ([AccountName] IS NULL OR [AccountName] = '') AND ([IsDefault] IS NULL OR [IsDefault] = 0) AND ([IsActive] IS NULL OR [IsActive] = 0)
			AND [WhiteList] = NULL AND [WhiteListDate] = NULL AND ([SWIFT_BIC] IS NULL OR [SWIFT_BIC] = '') AND ([BankName] IS NULL OR [BankName] = '')
			AND ([IsElectronic] IS NULL OR [IsElectronic] = 0)AND ([IsVatAccount] IS NULL OR [IsVatAccount] = 0)
		--select * from [#tblBankAccounts]

		-- deserializacja danych kontrahenta z xml
		INSERT INTO [#tblContractor]
		SELECT 
			[Id], CASE WHEN [Guid] IS NULL OR [Guid] = '' OR [Guid] = '00000000-0000-0000-0000-000000000000' THEN NEWID() ELSE [GUID] END,
			[Name], [Shortcut], [Nip], [NipUE], [Pesel], [Regon], [LinkedUnit],
			(CASE WHEN [PersonKind] LIKE 'Osoba fizyczna (firma)' THEN 1 ELSE (CASE WHEN [PersonKind] = 'Osoba fizyczna' THEN 2 ELSE 0 END) END) AS [Subject],
			[VATPayerActive], [VIES],
			-- exAttribute
			[RequestMppTransfers], [PaymentFormId], [PayRegistryId], [SaleKindId], [PriceTypeId], [AccountantParameter], [IdFk], [PositionFk],
			-- additional data
			[CatalogId], [ContractorKindId], [MarkerId], (CASE WHEN [Status] IS NULL OR [Status] LIKE '' OR [Status] LIKE 'Aktywny' THEN 1 ELSE 0 END) AS [Status],
			-- sale conditions
			[IsLimitActive], [LimitCurrency], [LimitValue], [PriceNegotiation],
			-- main person
			[FirstName], [Surname]
		FROM 
			OPENXML(@handle, '/Contractor')
			WITH
			(
				[Id] INT 'Id',
				[Guid] NVARCHAR(36) 'Guid',
				[Name] NVARCHAR(150) 'Name',
				[Shortcut] NVARCHAR(50) 'Shortcut',
				[Nip] NVARCHAR(20) 'Nip',
				[NipUE] NVARCHAR(20) 'NipUE',
				[Pesel] NVARCHAR(15) 'Pesel',
				[Regon] NVARCHAR(20) 'Regon',
				[LinkedUnit] BIT 'LinkedUnit',
				[PersonKind] NVARCHAR(100) 'Subject',
				[VATPayerActive] BIT 'VatPayerActive',
				[VIES] BIT 'Vies',
				-- exAttribute
				[RequestMppTransfers] NVARCHAR(10) 'RequestMppTransfers', -- splitPayment
				[PaymentFormId] INT 'PaymentFormId', -- formaPlatnosci
				[PayRegistryId] INT 'PayRegistryId', -- rejestr
				[SaleKindId] INT 'SaleKindId', -- rodzajCeny
				[PriceTypeId] INT 'PriceTypeId', -- typCeny
				[AccountantParameter] NVARCHAR(20) 'AccountantParameter', -- parametrFK
				[IdFk] INT 'IdFk', -- idFK
				[PositionFk] INT 'PositionFk', -- pozycja FK
				-- additional data
				[CatalogId] NVARCHAR(100) 'CatalogId',
				[ContractorKindId] INT 'ContractorKindId',
				[MarkerId] INT 'MarkerId',
				[Status] NVARCHAR(10) 'Status',
				-- sale conditions
				[IsLimitActive] BIT 'IsLimitActive',
				[LimitCurrency] NVARCHAR(20) 'LimitCurrency',
				[LimitValue] FLOAT 'LimitValue',
				[PriceNegotiation] BIT 'PriceNegotiation',
				-- main person
				[FirstName] NVARCHAR(50) 'FirstName',
				[Surname] NVARCHAR(50) 'SurName'
			)
		--select * from [#tblContractor]

		-- ustawienie domylnego znacznika; wpp. bug 195961
		UPDATE [#tblContractor]
		   SET [MarkerId] = (SELECT [ElementId] FROM [SSCommon].[STElements] WHERE [Guid] = 'D205CFCC-1AF4-4E6B-919C-9AE3A2192F81')
		 WHERE 0 = ISNULL([MarkerId], 0);

		-- deserializacja pl wasnych kontrahenta z xml
		DECLARE @classificationTableGuid UNIQUEIDENTIFIER
		DECLARE @classificationTableName NVARCHAR(255)
		SELECT 
			@classificationTableName = ISNULL([TableNamePrefix], '') + [TableName],
			@classificationTableGuid = [Guid]
		FROM [SSCommon].[STClasifications] 
		WHERE [TypeTable] LIKE 'Contractors' AND [Owner] = @moduleCode

		IF @classificationTableName IS NULL -- jeeli nie pobrano na podstawie kodu moduu to ustaw nazw wsplnej tabeli kontrahentw
			SET @classificationTableName = 'STContractorsClassification'

		DECLARE @customFieldsDefinitions TABLE([ColumnName] NVARCHAR(128),[Symbol] NVARCHAR(128),[Type] NVARCHAR(250))
		INSERT INTO @customFieldsDefinitions
		SELECT
			[Common].[GetNormalizedColumnNameForCustomField](dim.[Symbol]),
			[Common].[GetNormalizedSymbolForCustomField](dim.[Symbol]),
			dim.[DimCLRType]
		from [SSCommon].[STClasifications] clas
		INNER JOIN [SSCommon].[STClasificationDims] clasDim ON clasDim.[ClasificationsDefId] = clas.[ClasificationsDefId]
		INNER JOIN [SSCommon].[STDimensions] dim ON dim.[Id] = clasDim.[DimensionsId]
		where clas.[Guid] like @classificationTableGuid
		--SELECT * FROM @customFieldsDefinitions

		-- czy s zdefiniowane pola wasne
		IF EXISTS (SELECT * FROM @customFieldsDefinitions)
		BEGIN
			SELECT * INTO #temporaryContractorData
			FROM OPENXML(@handle, '/Contractor')
			;WITH documentFields as (
				SELECT c0.[parentid] AS [Id], c0.[localname] AS [FieldName], c1.[text] AS [Value]
				FROM #temporaryContractorData c0
				INNER JOIN #temporaryContractorData c1 ON c0.[id] = c1.[parentid]
				WHERE c0.[parentid] = 0 AND c0.[nodetype] = 1 AND c1.[nodetype] = 3 -- nodetype	int	Identifies the node type. 1 = Element node, 2 = Attribute node, 3 = Text node
			)
			INSERT INTO [#tblCustomFields]
			SELECT customFieldDefinitions.[Symbol] AS [FieldName], documentFields.[Value] AS [Value], customFieldDefinitions.[ColumnName] AS [ColumnName], customFieldDefinitions.[Type] AS [Type]
			FROM documentFields
			RIGHT OUTER JOIN @customFieldsDefinitions customFieldDefinitions ON customFieldDefinitions.[Symbol] = documentFields.[FieldName]

			IF (OBJECT_ID('tempdb..#temporaryContractorData') IS NOT NULL) DROP TABLE #temporaryContractorData
			--SELECT * FROM [#tblCustomFields]
		END

		DECLARE @contractorId INT
		SELECT @contractorId = [Id] FROM [#tblContractor]
		DECLARE @contactId INT
		DECLARE @contactGuid UNIQUEIDENTIFIER
		SET @contactGuid = NULL
		DECLARE @bankingInfoGuid UNIQUEIDENTIFIER
		SET @bankingInfoGuid = NULL
		IF ((@contactId IS NULL OR @contactId = 0) AND @contractorId IS NOT NULL AND @contractorId <> 0)
		BEGIN
			SELECT
				@contactGuid = ctr.[ContactGuid],
				@contactId = ct.[Id],
				@bankingInfoGuid = [BankingInfoGuid]
			FROM [SSCommon].[STContractors] ctr
			INNER JOIN [SSCommon].[STContacts] ct ON ct.[Guid] = ctr.[ContactGuid]
			WHERE ctr.[Id] = @contractorId
		END

-- zapis informacji do tabeli z danymi kontaktowymi
		-- tabela tymczasowa na TeleItContacts
		CREATE TABLE [#TeleItContacts]
		(
			[Id] INT IDENTITY(1,1),
			[Guid] UNIQUEIDENTIFIER,
			[Name] NVARCHAR(50),
			[ContactGuid] UNIQUEIDENTIFIER,
			[Telephone1] NVARCHAR(50),
			[Telephone2] NVARCHAR(50),
			[Telephone3] NVARCHAR(50),
			[Fax] NVARCHAR(50),
			[Fax2] NVARCHAR(50),
			[Telex] NVARCHAR(50),
			[Email1] NVARCHAR(100),
			[Email2] NVARCHAR(100),
			[WWW] NVARCHAR(100),
			[Communicator1] NVARCHAR(50),
			[Communicator2] NVARCHAR(50),
			[Description] NVARCHAR(3000),
			[Facebook] NVARCHAR(100)
		)

		-- tabela tymczasowa na PostOfficeAddresses
		CREATE TABLE [#PostOfficeAddresses]
		(
			[Id] INT IDENTITY(1,1),
			[AddressName] NVARCHAR(64),
			[Guid] UNIQUEIDENTIFIER,
			[Street] NVARCHAR(50),
			[Commune] NVARCHAR(50),
			[District] NVARCHAR(50),
			[Place] NVARCHAR(50),
			[PostCode] NVARCHAR(10),
			[PostOffice] NVARCHAR(50),
			[ApartmentNo] NVARCHAR(15),
			[HouseNo] NVARCHAR(15),
			[ContactGuid] UNIQUEIDENTIFIER,
			[Province] NVARCHAR(50),
			[Country] NVARCHAR(50),
			[PostOfficeBox] NVARCHAR(50)
		)

		DECLARE @contactName NVARCHAR(100) = ''
		DECLARE @contactMainPostOfficeAddress UNIQUEIDENTIFIER
		DECLARE @contactMainTeleITContact UNIQUEIDENTIFIER
		DECLARE @contactPostOfficeAddresses NVARCHAR(max)
		DECLARE @contactTeleItContacts NVARCHAR(max)
		DECLARE @contactWriteAction INT = 0

		IF EXISTS (SELECT * FROM [#tblContact])
		BEGIN
			
			IF (@contactId IS NULL OR @contactId = 0) -- nowy kontakt
			BEGIN
				IF (@contactGuid IS NULL OR @contactGuid LIKE '00000000-0000-0000-0000-000000000000')
					SELECT @contactGuid = CASE WHEN [Guid] IS NOT NULL AND [Guid] NOT LIKE '00000000-0000-0000-0000-000000000000' THEN [Guid] ELSE NEWID() END FROM [#tblContact]

				SELECT 
					@contactName = '',
					@contactMainTeleITContact = NEWID(),
					@contactWriteAction = 1
			END
			ELSE -- istniejcy kontakt
			BEGIN
				SELECT 
					@contactName = [Name], 
					@contactWriteAction = 2
				FROM [SSCommon].[STContacts] 
				WHERE [Guid] = @contactGuid

				SELECT @contactMainTeleITContact = [Guid] FROM [SSCommon].[STTeleItContacts] WHERE [ContactGuid] LIKE @contactGuid
				IF (@contactMainTeleITContact IS NULL)
					SET @contactMainTeleITContact = NEWID()
			END
			INSERT INTO [#TeleItContacts]([Guid], [Name], [ContactGuid], [Telephone1], [Telephone2], [Telephone3], [Fax], [Fax2], [Email1], [Email2], [WWW],
				[Communicator1], [Communicator2], [Description], [Telex], [Facebook])
			SELECT @contactMainTeleITContact, 'adres domylny', @contactGuid, ISNULL(c.[Telephone1], ''), ISNULL(c.[Telephone2], ''), '', ISNULL(c.[Fax], ''), '', ISNULL(c.[Email], ''), '', ISNULL(c.[Website], ''),
				'', '', '', ISNULL(c.[Telex], ''), ISNULL(c.[Facebook], '')
			FROM [#tblContact] c
		END

		-- zapis informacji do tabeli z danymi adresu domylnego
		IF EXISTS (SELECT * FROM [#tblMainAddress])
		BEGIN
			IF (@contactGuid IS NULL)
				SET @contactGuid = NEWID()

			DECLARE @postOfficeAddressName NVARCHAR(64)
			SELECT @contactMainPostOfficeAddress = [Guid], @postOfficeAddressName = [Name] FROM [#tblMainAddress]

			IF (@contactId IS NULL OR @contactId = 0) -- nowy kontakt
			BEGIN
				SELECT @contactId = [Id] FROM [SSCommon].[STContacts] WHERE [Guid] LIKE @contactGuid
				IF (@contactMainPostOfficeAddress IS NULL OR @contactMainPostOfficeAddress LIKE '00000000-0000-0000-0000-000000000000')
					SET @contactMainPostOfficeAddress = NEWID()
			END
			ELSE -- istniejcy kontakt
			BEGIN
				IF EXISTS(SELECT * FROM [SSCommon].[STPostOfficeAddresses] WHERE [ContactGuid] LIKE @contactGuid AND [AddressName] LIKE @postOfficeAddressName)
					SELECT @contactMainPostOfficeAddress = [Guid] FROM [SSCommon].[STPostOfficeAddresses] WHERE [ContactGuid] LIKE @contactGuid AND [AddressName] LIKE @postOfficeAddressName
				ELSE
					IF (@contactMainPostOfficeAddress IS NULL OR @contactMainPostOfficeAddress LIKE '00000000-0000-0000-0000-000000000000')
						SET @contactMainPostOfficeAddress = NEWID()
			END
			INSERT INTO [#PostOfficeAddresses]([AddressName], [Guid], [Street], [Commune], [District], [Place], [PostCode], [PostOffice], [ApartmentNo],
				[HouseNo], [ContactGuid], [Province], [Country], [PostOfficeBox])
			SELECT CASE WHEN a.[Name] IS NULL OR a.[Name] = '' THEN 'adres domylny' ELSE a.[Name] END, @contactMainPostOfficeAddress, a.[Street], '', '', a.[Place], a.[PostalCode], a.[Post], a.[ApartmentNo],
				a.[HouseNo], @contactGuid, a.[Province], a.[Country], ''
			FROM [#tblMainAddress] a
		END	

		-- zapis informacji do tabeli z danymi adresu korespondencyjnego
		IF EXISTS (SELECT * FROM [#tblCorespondenceAddress])
		BEGIN
			DECLARE @corespondenceAddressId INT
			DECLARE @corespondenceAddressGuid UNIQUEIDENTIFIER
			DECLARE @corespondenceAddressName NVARCHAR(64)
			SELECT  @corespondenceAddressId = [Id], @corespondenceAddressGuid = [Guid], @corespondenceAddressName = [Name] FROM [#tblCorespondenceAddress]

			IF (@corespondenceAddressId IS NULL OR @corespondenceAddressId = 0) -- nowy adres korespondencyjny
				IF (@corespondenceAddressGuid IS NULL OR @corespondenceAddressGuid LIKE '00000000-0000-0000-0000-000000000000')
					SET @corespondenceAddressGuid = NEWID()	
			ELSE
			BEGIN
				IF EXISTS (SELECT * FROM [SSCommon].[STPostOfficeAddresses] WHERE [ContactGuid] LIKE @contactGuid AND [AddressName] LIKE @corespondenceAddressName)
					SELECT @corespondenceAddressGuid = [Guid] FROM [SSCommon].[STPostOfficeAddresses] WHERE [ContactGuid] LIKE @contactGuid AND [AddressName] LIKE @corespondenceAddressName
				ELSE
					IF (@corespondenceAddressGuid IS NULL OR @corespondenceAddressGuid LIKE '00000000-0000-0000-0000-000000000000')
						SET @corespondenceAddressGuid = NEWID()
			END
			INSERT INTO [#PostOfficeAddresses]([AddressName], [Guid], [Street], [Commune], [District], [Place], [PostCode], [PostOffice], [ApartmentNo],
				[HouseNo], [ContactGuid], [Province], [Country], [PostOfficeBox])
			SELECT CASE WHEN a.[Name] IS NULL OR a.[Name] = '' THEN 'adres korespondencyjny' ELSE a.[Name] END, @corespondenceAddressGuid, a.[Street], '', '', a.[Place], a.[PostalCode], a.[Post], a.[ApartmentNo],
				a.[HouseNo], @contactGuid, a.[Province], a.[Country], ''
			FROM [#tblCorespondenceAddress] a
		END

		-- odczytanie pozostaych adresw podpietych pod kontakt (np. adresy dostaw)
		INSERT INTO [#PostOfficeAddresses]([AddressName], [Guid], [Street], [Commune], [District], [Place], [PostCode], [PostOffice], [ApartmentNo],
			[HouseNo], [ContactGuid], [Province], [Country], [PostOfficeBox])
		SELECT [AddressName], [Guid], [Street], [Commune], [District], [Place], [PostCode], [PostOffice], [ApartmentNo],
			[HouseNo], [ContactGuid], [Province], [Country], [PostOfficeBox]
		FROM [SSCommon].[STPostOfficeAddresses]
		WHERE [ContactGuid] = @contactGuid AND [AddressName] NOT LIKE 'adres domylny' AND [AddressName] NOT LIKE 'adres korespondencyjny'


		SET @contactTeleItContacts = (SELECT * FROM #TeleItContacts FOR XML PATH('TeleItContacts'), root ('DocumentElement'))
		SET @contactPostOfficeAddresses = (SELECT * FROM #PostOfficeAddresses FOR XML PATH('PostOfficeAddresses'), root ('DocumentElement'))
		
		EXEC [SSCommon].[sp_SaveContactWithDetails] 
					@contactId,
					@contactGuid,
					@contactName,
					@contactMainPostOfficeAddress,
					@contactMainTeleITContact,
					@contactPostOfficeAddresses,
					@contactTeleItContacts,
					@contactWriteAction,
					@contextInfo

-- zapis informacji do tabeli z danymi bankowymi
-- TODO wykorzysta procedur sp_InsertBankingInfoAndAccounts, sp_UpdateBankingInfoAndAccounts, sp_InsertElementAndAttrs, sp_SaveElementAndAttrs, sp_UpdateElementAndAttrs jeeli jest taka moliwo
		DECLARE @removingElementsTable TABLE([ElementId] INT, [AccountId] INT) -- tymczasowa tabela na elementy do usunicia
		IF EXISTS (SELECT * FROM [#tblBankAccounts])
		BEGIN	
			-- tabela tymczasowa na Rachunki bankowe
			CREATE TABLE [#Accounts]
			(
				[Id] INT IDENTITY(1,1),
				[Guid] UNIQUEIDENTIFIER,
				[BankingInfosGuid] UNIQUEIDENTIFIER,
				[AccountNo] NVARCHAR(50),
				[BankName] NVARCHAR(100),
				[AccountName] NVARCHAR(100),
				[ElementId] INT,
				[MainElementGuid] UNIQUEIDENTIFIER,
				[SWIFT_BIC] NVARCHAR(10),
				[Active] BIT,
				[WhiteList] BIT,
				[WhiteListDate] DATETIME2
			)

			DECLARE @bankAccountsEKId INT
			DECLARE @bankAccountsEKGuid UNIQUEIDENTIFIER = '4849512E-9024-4B7C-BD06-737BF4FF3A77'
			SELECT @bankAccountsEKId = [ElementKindId] FROM [SSCommon].[STElementKinds] WHERE [Guid] LIKE @bankAccountsEKGuid
			DECLARE @minBankAccountsPosition INT
			DECLARE @bankingId INT
			DECLARE @bankingMainElement UNIQUEIDENTIFIER
			DECLARE @bankingAccounts NVARCHAR(MAX)

			-- sprawdzenie czy kontrahent ma BankingInfo
			IF (@bankingInfoGuid IS NULL) -- nowe banking info
			BEGIN
				SET @bankingInfoGuid = NEWID()
				EXEC [SSCommon].[sp_InsertBankingInfoAndAccounts] 
					0,
					@bankingInfoGuid,
					'',
					'<DocumentElement />',
					@contextInfo,
					NULL
			END
			
			-- usunicie kont bankowych
			INSERT INTO @removingElementsTable
			SELECT cmnElements.[ElementId], cmnAccounts.[Id] FROM [SSCommon].[STElements] cmnElements
			INNER JOIN [SSCommon].[STAccounts] cmnAccounts ON cmnAccounts.[MainElementGuid] = cmnElements.[Guid]
			LEFT OUTER JOIN [#tblBankAccounts] tAccount ON tAccount.[Id] = cmnAccounts.[Id]
			WHERE tAccount.[Id] IS NULL AND cmnAccounts.[BankingInfosGuid] LIKE @bankingInfoGuid

			IF EXISTS(SELECT * FROM @removingElementsTable)
			BEGIN
				DELETE FROM [SSCommon].[STElementExAttributes]
				WHERE [ElementId] IN (SELECT [ElementId] FROM @removingElementsTable)

				UPDATE [SSCommon].[STBankingInfos]
				SET [MainAccount] = NULL
				FROM [SSCommon].[STBankingInfos] bi
				INNER JOIN #tblBankAccounts tba ON bi.[MainAccount] = tba.[Guid]
				WHERE bi.[Guid] = @bankingInfoGuid
			
				BEGIN TRY
					DELETE FROM [SSCommon].[STAccounts]
					WHERE [Id] IN (SELECT [AccountId] FROM @removingElementsTable)
			
					DELETE FROM [SSCommon].[STElements]
					WHERE [ElementId] IN (SELECT [ElementId] FROM @removingElementsTable)
				END TRY
				BEGIN CATCH
					IF (ERROR_NUMBER() = 547)
						THROW 50000, N'Nie mona usun rachunku bankowego. Istniej dokumenty powizane z danym rachunkiem.', 1
					ELSE
						THROW 50000, N'Nie powiodo si usuwanie rachunkw bankowych z bazy danych', 1
				END CATCH
			END		
			-- aktualizacja rachunkw bankowych
			SELECT @minBankAccountsPosition = ISNULL(MIN([Position]), 0) FROM [SSCommon].[STElements] WHERE [ElementKindId] = @bankAccountsEKId
		
			BEGIN TRY
				--aktualizacja STElements
				SELECT
						@extAttAccountNo = '',
						@extAttDescription = '',
						@extAttExtAttributes = '<DocumentElement />',					
						@extAttExtOtherThenOriginal=0			
				
				DECLARE @counter INT = 0
				
				DECLARE @accountToHandle NVARCHAR(50)
				SELECT @accountToHandle = [Guid]
				FROM #tblBankAccounts
				ORDER BY [Id] ASC
				OFFSET @counter ROWS
				FETCH NEXT 1 ROWS ONLY

				Declare @v nvarchar(max) = (select * from #tblBankAccounts
				ORDER BY [Guid] ASC for xml auto)

				WHILE @accountToHandle IS NOT NULL
				BEGIN
					SELECT
						@extAttElementId = CASE WHEN elem.[ElementId] IS NULL OR elem.[ElementId] = 0 THEN -1 ELSE elem.[ElementId] END,
						@extAttTitle = tAcc.[Guid],
						@extAttActive = tAcc.[IsActive],
						@extAttShortcut = tAcc.[Guid],
						@extAttPosition = @minBankAccountsPosition - 1,
						@minBankAccountsPosition = @minBankAccountsPosition - 1,
						@extAttExtWriteAction = CASE WHEN elem.[ObjVer] IS NULL THEN 1 ELSE 2 END,
						@extAttExtObjVer = CASE WHEN elem.[ElementId] IS NULL OR elem.[ElementId] = 0 THEN 0 ELSE elem.[ObjVer] + 1 END
					FROM #tblBankAccounts tAcc
					LEFT OUTER JOIN [SSCommon].[STElements] elem ON tAcc.[Guid] = elem.[Guid]
					WHERE tAcc.[Guid] = @accountToHandle

					EXEC [SSCommon].[sp_SaveElementAndAttrs] 
							@extAttElementId , 
							@accountToHandle, 
							@extAttTitle,
							@extAttActive,
							@extAttAccountNo,
							@extAttDescription,
							@bankAccountsEKId,
							@extAttShortcut,
							@extAttPosition,
							@extAttExtObjVer,
							@extAttExtAttributes,
							@bankAccountsEKGuid,
							@extAttExtWriteAction,
							@extAttExtOtherThenOriginal,
							@contextInfo
					
					SET @accountToHandle = NULL
					SELECT @accountToHandle = [Guid],
						@counter = @counter + 1
					FROM #tblBankAccounts
					ORDER BY [Id] ASC
					OFFSET @counter ROWS
					FETCH NEXT 1 ROWS ONLY
				END

				SELECT @bankingMainElement = [Guid]
				FROM #tblBankAccounts
				WHERE [IsDefault] = 1
				--aktualizacja danych konkretnych rachunkw bankowych
				INSERT INTO #Accounts ([Guid], [BankingInfosGuid], [AccountNo], [BankName],
					[Active], [SWIFT_BIC], [WhiteList], [WhiteListDate])
				SELECT 
					CASE WHEN tAccount.[Guid] IS NULL OR tAccount.[Guid] LIKE '00000000-0000-0000-0000-000000000000' THEN NEWID() ELSE tAccount.[Guid] END,
					@bankingInfoGuid, tAccount.[AccountNo], ISNULL(tAccount.[BankName], ''),
					ISNULL(tAccount.[IsActive], 1), ISNULL(tAccount.[SWIFT_BIC], ''),
					tAccount.[WhiteList], tAccount.[WhiteListDate]
				FROM #tblBankAccounts tAccount
				LEFT OUTER JOIN [SSCommon].[STAccounts] sa ON tAccount.[Guid] = sa.[Guid]
				
				UPDATE #Accounts
				SET [AccountName] = [Guid],
					[MainElementGuid] = [Guid]
				FROM #Accounts
				
				SET @bankingAccounts = (SELECT * FROM #Accounts FOR XML PATH('Accounts'), root ('DocumentElement'))
			
				EXEC [SSCommon].[sp_UpdateBankingInfoAndAccounts] 
					@bankingId,
					@bankingInfoGuid,
					'',
					@bankingAccounts,
					@contextInfo,
					@bankingMainElement

			END TRY
			BEGIN CATCH
				THROW 50000, N'Nie powioda si aktualizacja rachunkw bankowych', 1
			END CATCH
		END
		ELSE -- jeeli istniay rachunki bankowe to zostay usunite (UWAGA: przypisane rachunki bankowe np. do dokumentw nie zostan usunite)
		BEGIN 
			IF (@bankingInfoGuid IS NOT NULL AND @bankingInfoGuid NOT LIKE '00000000-0000-0000-0000-000000000000')
			BEGIN
				BEGIN TRY
				-- usunicie kont bankowych
				INSERT INTO @removingElementsTable
				SELECT cmnElements.[ElementId], cmnAccounts.[Id] FROM [SSCommon].[STElements] cmnElements
				INNER JOIN [SSCommon].[STAccounts] cmnAccounts ON cmnAccounts.[MainElementGuid] = cmnElements.[Guid]
				LEFT OUTER JOIN [#tblBankAccounts] tAccount ON tAccount.[Id] = cmnAccounts.[Id]
				WHERE tAccount.[Id] IS NULL AND cmnAccounts.[BankingInfosGuid] LIKE @bankingInfoGuid
				
				IF EXISTS(SELECT * FROM @removingElementsTable)
				BEGIN
					DELETE FROM [SSCommon].[STElementExAttributes]
					WHERE [ElementId] IN (SELECT [ElementId] FROM @removingElementsTable)

					UPDATE [SSCommon].[STBankingInfos]
					SET [MainAccount] = NULL
					WHERE [Guid] = @bankingInfoGuid
				
					DELETE FROM [SSCommon].[STAccounts]
					WHERE [Id] IN (SELECT [AccountId] FROM @removingElementsTable)
				
					DELETE FROM [SSCommon].[STElements]
					WHERE [ElementId] IN (SELECT [ElementId] FROM @removingElementsTable)
		
					UPDATE [SSCommon].[STContractors]
					SET [BankingInfoGuid] = NULL
					WHERE [Id] = @contractorId

					DELETE FROM [SSCommon].[STBankingInfos]
					WHERE [Guid] LIKE @bankingInfoGuid

					SET @bankingInfoGuid = NULL
				END
				END TRY
				BEGIN CATCH
					IF (ERROR_NUMBER() = 547)
						THROW 50000, N'Nie mona usun rachunku bankowego. Istniej dokumenty powizane z danym rachunkiem.', 1
					ELSE
						THROW 50000, N'Nie powiodo si usuwanie rachunkw bankowych z bazy danych', 1
				END CATCH
			END
		END

		IF EXISTS (SELECT * FROM [#tblContractor])
		BEGIN
			DECLARE @contractorMainElement UNIQUEIDENTIFIER
			DECLARE @mainPersonGuid UNIQUEIDENTIFIER
			SET @mainPersonGuid = NULL
			DECLARE @contractorElementKindId INT
			SELECT @contractorElementKindId = [ELementKindId] FROM [SSCommon].[STElementKinds] WHERE [Guid] LIKE 'BC090993-0768-4487-BE6F-E44E9501734C'
			DECLARE @firstname NVARCHAR(50), @surname NVARCHAR(50)
			SELECT @firstname = [FirstName], @surname = [Surname] FROM [#tblContractor]
			IF (@firstname IS NULL OR @firstname LIKE '')
				SELECT @firstname = [FirstName] FROM [#tblContact]
			IF (@surname IS NULL OR @surname LIKE '')
				SELECT @surname = [Surname] FROM [#tblContact]

			IF (@contractorId IS NULL OR @contractorId = 0)
			BEGIN
				SET @contractorMainElement = NEWID()

				IF EXISTS(SELECT * FROM [SSCommon].[STElements] elem INNER JOIN [#tblContractor] tContractor ON elem.[Shortcut] LIKE tContractor.[Shortcut] INNER JOIN [SSCommon].[STElementKinds] ekinds ON elem.[ElementKindId] = ekinds.[ElementKindId] WHERE ekinds.[Name] LIKE 'Kontrahenci')
					THROW 50000, N'Kontrahent o podanym skrcie istnieje w bazie danych', 1

				BEGIN TRY	
					SELECT 
						@extAttAccountNo = 'COMDA#RESETVALIDACCOUNTNO2008', 
						@extAttDescription = '',
						@extAttPosition = NULL,
						@extAttExtObjVer = 0,
						@extAttExtWriteAction = 1,
						@extAttExtOtherThenOriginal = 0
					
					IF @moduleCode LIKE 'FKF'
					BEGIN
						SELECT 
							@extAttAccountNo = 'COMDA#GENERATEVALIDACCOUNTNO2008', 
							@extAttPosition = (SELECT [PositionFk] FROM #tblContractor WHERE [Id] = @contractorId)
					END

					SELECT @extAttTitle = tContractor.[Name], 
						@extAttActive = ISNULL(tContractor.[Status], 1), 						
						@extAttShortcut = ISNULL(tContractor.[Shortcut], '')
					FROM [#tblContractor] tContractor
					
					SELECT @extAttElementId = elem.[ElementId]
					FROM [SSCommon].[STElements] elem
					WHERE elem.[Guid] = @contractorMainElement

					SELECT @extAttExtElementKindGuid = kind.[Guid]
					FROM [SSCommon].[STElementKinds] kind
					INNER JOIN [SSCommon].[STElements] elem ON kind.[ElementKindId] = elem.[ElementKindId]
					WHERE elem.[ElementId] = @extAttElementId

					-- exAttributes
					-- splitPayment
					INSERT INTO #ExtAttributes([ElementId], [Name], [StringValue],
						[OriginalCLRType], [Guid], [Title], [InheritedFlag], [AccessFlag])
					SELECT @extAttElementId, 'splitPayment', (SELECT CASE WHEN [RequestMppTransfers] LIKE 'Wymagaj' THEN 2 ELSE (CASE WHEN [RequestMppTransfers] LIKE 'Blokada' THEN 1 ELSE 0 END) END FROM [#tblContractor]),
						'System.Int16, mscorlib, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089', NEWID(), 'splitPayment', 0, 0
					-- idFK
					INSERT INTO #ExtAttributes([ElementId], [Name], [StringValue],
						[OriginalCLRType], [Guid], [Title], [InheritedFlag], [AccessFlag])
					SELECT @extAttElementId, 'idFK', (SELECT CASE WHEN [IdFk] = 0 THEN '' ELSE CAST([IdFk] AS NVARCHAR(1024)) END FROM [#tblContractor]),
						'System.String, mscorlib, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089', NEWID(), 'idFK', 0, 0						
					-- parametrFK
					INSERT INTO #ExtAttributes([ElementId], [Name], [StringValue],
						[OriginalCLRType], [Guid], [Title], [InheritedFlag], [AccessFlag])
					SELECT @extAttElementId, 'parametrFK', (SELECT ISNULL([AccountantParameter], '') FROM [#tblContractor]),
						'System.String, mscorlib, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089', NEWID(), 'parametrFK', 0, 0
					-- typCeny
					INSERT INTO #ExtAttributes([ElementId], [Name], [StringValue],
						[OriginalCLRType], [Guid], [Title], [InheritedFlag], [AccessFlag])
					SELECT @extAttElementId, 'typCeny', (SELECT ISNULL([PriceTypeId], 0) FROM [#tblContractor]),
						'System.String, mscorlib, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089', NEWID(), 'typCeny', 0, 0
					-- rodzajCeny
					INSERT INTO #ExtAttributes([ElementId], [Name], [StringValue],
						[OriginalCLRType], [Guid], [Title], [InheritedFlag], [AccessFlag])
					SELECT @extAttElementId, 'rodzajCeny', (SELECT ISNULL([SaleKindId],0) FROM [#tblContractor]),
						'System.String, mscorlib, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089', NEWID(), 'rodzajCeny', 0, 0
					-- rejestr
					INSERT INTO #ExtAttributes([ElementId], [Name], [StringValue],
						[OriginalCLRType], [Guid], [Title], [InheritedFlag], [AccessFlag])
					SELECT @extAttElementId, 'rejestr', (SELECT ISNULL([PayRegistryId], 0) FROM [#tblContractor]),
						'System.String, mscorlib, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089', NEWID(), 'rejestr', 0, 0
					-- formaPlatnosci
					INSERT INTO #ExtAttributes([ElementId], [Name], [StringValue],
						[OriginalCLRType], [Guid], [Title], [InheritedFlag], [AccessFlag])
					SELECT @extAttElementId, 'formaPlatnosci', (SELECT ISNULL([PaymentFormId], 0) FROM [#tblContractor]),
						'System.String, mscorlib, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089', NEWID(), 'formaPlatnosci', 0, 0
  
					SET @extAttExtAttributes = (SELECT * FROM #ExtAttributes FOR XML PATH('ElementExAttributes'), root ('DocumentElement'))
					
					EXEC [SSCommon].[sp_SaveElementAndAttrs] 
						@extAttElementId , 
						@contractorMainElement, 
						@extAttTitle,
						@extAttActive,
						@extAttAccountNo,
						@extAttDescription,
						@contractorElementKindId,
						@extAttShortcut,
						@extAttPosition,
						@extAttExtObjVer,
						@extAttExtAttributes,
						@extAttExtElementKindGuid,
						@extAttExtWriteAction,
						@extAttExtOtherThenOriginal,
						@contextInfo

					IF ((@surname IS NOT NULL AND @surname NOT LIKE '') OR (@firstname IS NOT NULL AND @firstname NOT LIKE ''))
					BEGIN
						SET CONTEXT_INFO @ContextInfoBinaryP;
						SET @mainPersonGuid = NEWID()
						INSERT INTO [SSCommon].[STPersons]([Firstname],[SecondName],[Surname],[BirthDate],[Guid],[StringIdent],
							[DualStringIdent1],[DualStringIdent2],[Note],[ContactGuid],[BankingInfoGuid],[DataSourceName],[DataSourceAddress],[DataProcessingAgree],[DataMarketingProcessingAgree],
							[MainElement],[ExcludedFromAnonymization],[Anonymized])
						SELECT ISNULL(@firstname, ''), '', ISNULL(@surname, ''), CAST('1753-01-01 00:00:00.000' AS DATETIME2), @mainPersonGuid, ISNULL(tContractor.[Name], ''),
							ISNULL(tContractor.[Pesel],''), ISNULL(tContractor.[Nip],''), '', @contactGuid, @bankingInfoGuid, '', '', 0, 0,
							@contractorMainElement, 0, 0
						FROM [#tblContractor] tContractor

						INSERT INTO [SSCommon].[STPersonsClassification]([Guid], [ElementId], [CDim_UserPerson], [CDim_EmployeePerson], [CDim_ContractorContactPerson])
							VALUES(NEWID(), @mainPersonGuid, 0, 0, 1)
						SET CONTEXT_INFO @ContextInfoBinary;
					END

					INSERT INTO [SSCommon].[STContractors]([Guid],
						[Shortcut],[Name],[NIP],[PESEL],[REGON],[NIP_UE],
						[ContactGuid],[BankingInfoGuid],[LimitFlag],[LimitAmount],[LimitCurrency],
						[VIES],[Negotiation],[MainElement],[MainPerson],[VATPayerActive],
						[NaturalPerson],[LinkedUnit])
					SELECT
						CASE WHEN tContractor.[Guid] IS NULL OR tContractor.[Guid] LIKE '00000000-0000-0000-0000-000000000000' THEN NEWID() ELSE tContractor.[Guid] END,
						tContractor.[Shortcut], tContractor.[Name], ISNULL(tContractor.[Nip], ''), ISNULL(tContractor.[Pesel], ''), ISNULL(tContractor.[Regon], ''), ISNULL(tContractor.[NipUE], ''),
						@contactGuid, @bankingInfoGuid, ISNULL(tContractor.[IsLimitActive], 0), ISNULL(tContractor.[LimitValue], 0), ISNULL(tContractor.[LimitCurrency], ''),
						ISNULL(tContractor.[VIES], 0), ISNULL(tContractor.[PriceNegotiation], 0), @contractorMainElement, @mainPersonGuid, ISNULL(tContractor.[VATPayerActive], 0),
						ISNULL(tContractor.[PersonKind], 0), ISNULL(tContractor.[LinkedUnit], 0)
					FROM [#tblContractor] tContractor

					SELECT @contractorId = [Id] FROM [SSCommon].[STContractors] WHERE Shortcut = @extAttShortcut
				END TRY
				BEGIN CATCH	
					THROW 50000, N'Nie powiodo si dodawanie kontrahenta', 1
				END CATCH
			END
			ELSE
			BEGIN
				BEGIN TRY
					SELECT @mainPersonGuid = [MainPerson], @contractorMainElement = [MainElement] FROM [SSCommon].[STContractors] WHERE [Id] = @contractorId
					
					SET CONTEXT_INFO @ContextInfoBinaryP;

					IF ((@surname IS NOT NULL AND @surname NOT LIKE '') OR (@firstname IS NOT NULL AND @firstname NOT LIKE ''))
					BEGIN
						IF (@mainPersonGuid IS NULL)
						BEGIN
							SET @mainPersonGuid = NEWID()

							INSERT INTO [SSCommon].[STPersons]([Firstname],[SecondName],[Surname],[BirthDate],[Guid],[StringIdent],
								[DualStringIdent1],[DualStringIdent2],[Note],[ContactGuid],[BankingInfoGuid],[DataSourceName],[DataSourceAddress],[DataProcessingAgree],[DataMarketingProcessingAgree],
								[MainElement],[ExcludedFromAnonymization],[Anonymized])
							SELECT ISNULL(@firstname, ''), '', ISNULL(@surname, ''), CAST('1753-01-01 00:00:00.000' AS DATETIME2), @mainPersonGuid, tContractor.[Name],
								ISNULL(tContractor.[Pesel],''), ISNULL(tContractor.[Nip],''), '', @contactGuid, @bankingInfoGuid, '', '', 0, 0,
								@contractorMainElement, 0, 0
							FROM [#tblContractor] tContractor
														
							INSERT INTO [SSCommon].[STPersonsClassification]([Guid], [ElementId], [CDim_UserPerson], [CDim_EmployeePerson], [CDim_ContractorContactPerson])
								VALUES(NEWID(), @mainPersonGuid, 0, 0, 1)
						END
						ELSE
						BEGIN
							UPDATE [SSCommon].[STPersons]
							SET [Firstname] = ISNULL(@firstname, ''), [Surname] = ISNULL(@surname, '')
							FROM [#tblContractor] tContractor
							WHERE [SSCommon].[STPersons].Guid LIKE @mainPersonGuid
						END
					END
					ELSE
					BEGIN
						DELETE FROM [SSCommon].[STPersons]
						WHERE [Guid] LIKE @mainPersonGuid

						SET @mainPersonGuid = NULL
					END			
					
					SET CONTEXT_INFO @ContextInfoBinary;

					UPDATE [SSCommon].[STContractors]
					SET
						[Shortcut] = ISNULL(tContractor.[Shortcut], ''),
						[Name] = ISNULL(tContractor.[Name], ''),
						[NIP] = ISNULL(tContractor.[Nip], ''),
						[PESEL] = ISNULL(tContractor.[Pesel], ''),
						[REGON] = ISNULL(tContractor.[Regon], ''),
						[NIP_UE] = ISNULL(tContractor.NipUE, ''),
						[ContactGuid] = @contactGuid,
						[BankingInfoGuid] = @bankingInfoGuid,
						[LimitFlag] = ISNULL(tContractor.[IsLimitActive], 0),
						[LimitAmount] = ISNULL(tContractor.[LimitValue], 0),
						[LimitCurrency] = ISNULL(tContractor.[LimitCurrency], ''),
						[VIES] = ISNULL(tContractor.[VIES], 0),
						[Negotiation] = ISNULL(tContractor.[PriceNegotiation], 0),
						[MainPerson] = @mainPersonGuid,
						[VATPayerActive] = ISNULL(tContractor.[VATPayerActive], 0),
						[NaturalPerson] = ISNULL(tContractor.[PersonKind], 0),
						[LinkedUnit] = ISNULL(tContractor.[LinkedUnit], 0)
					FROM [SSCommon].[STContractors] cmnCtrs
					INNER JOIN [#tblContractor] tContractor ON tContractor.[Id] = cmnCtrs.[Id]
					WHERE cmnCtrs.[Id] = @contractorId 
												
					SELECT @extAttElementId = elem.[ElementId]
					FROM [SSCommon].[STElements] elem
					WHERE elem.[Guid] = @contractorMainElement
					
					SELECT 
						@extAttAccountNo = 'COMDA#UPDATEVALIDACCOUNTNO2008', 
						@extAttDescription = [Description],
						@extAttPosition = NULL,
						@extAttExtObjVer = [ObjVer] + 1,
						@extAttExtWriteAction = 2,
						@extAttExtOtherThenOriginal = 0
					FROM [SSCommon].[STElements] 
					WHERE [ElementId] = @extAttElementId					
					
					IF @moduleCode LIKE 'FKF'
						SET @extAttPosition = (SELECT [PositionFk] FROM #tblContractor WHERE [Id] = @contractorId)
					
					SELECT @extAttTitle = ISNULL(tContractor.[Name], ''), 
						@extAttActive = ISNULL(tContractor.[Status], 1), 						
						@extAttShortcut = ISNULL(tContractor.[Shortcut], '')
					FROM [#tblContractor] tContractor

					SELECT @extAttExtElementKindGuid = kind.[Guid]
					FROM [SSCommon].[STElementKinds] kind
					INNER JOIN [SSCommon].[STElements] elem ON kind.[ElementKindId] = elem.[ElementKindId]
					WHERE elem.[ElementId] = @extAttElementId

					-- exAttributes
					DECLARE @exAttributeId INT
					DECLARE @exAttributeGuid UNIQUEIDENTIFIER
					DECLARE @exAttributeCLRType NVARCHAR(400)
					DECLARE @exAttributeTitle NVARCHAR(200)
					DECLARE @exAttributeAccessFlag BIT
					DECLARE @exAttributeInheritedFlag BIT

					--	splitPayment
					SET @exAttributeId = NULL
					SELECT @exAttributeId = ex.[ElementExAttrId], @exAttributeGuid = ex.[Guid], @exAttributeCLRType = ex.[OriginalCLRType],
							@exAttributeTitle = ex.[Title], @exAttributeAccessFlag = ex.[AccessFlag], @exAttributeInheritedFlag = ex.[InheritedFlag]
						FROM [SSCommon].[STElementExAttributes] ex
						INNER JOIN [SSCommon].[STElements] elem ON elem.[ElementId] = ex.[ElementId]
						INNER JOIN [SSCommon].[STContractors] ctrs ON ctrs.[MainElement] = elem.[Guid]
						WHERE ctrs.[Id] = @contractorId AND ex.[Name] LIKE 'splitPayment'
					IF (@exAttributeId IS NOT NULL)
					BEGIN
						INSERT INTO #ExtAttributes([ElementId], [Name], [StringValue], 
							[OriginalCLRType], [Guid], [Title], [InheritedFlag], [AccessFlag])
						SELECT @exAttributeId, 'splitPayment', (SELECT CASE WHEN [RequestMppTransfers] LIKE 'Wymagaj' THEN 2 ELSE (CASE WHEN [RequestMppTransfers] LIKE 'Blokada' THEN 1 ELSE 0 END) END FROM [#tblContractor]),
							@exAttributeCLRType, @exAttributeGuid, @exAttributeTitle, @exAttributeInheritedFlag, @exAttributeAccessFlag
					END
					ELSE
					BEGIN					
						INSERT INTO #ExtAttributes([ElementId], [Name], [StringValue],
							[OriginalCLRType], [Guid], [Title], [InheritedFlag], [AccessFlag])
						SELECT @extAttElementId, 'splitPayment', (SELECT CASE WHEN [RequestMppTransfers] LIKE 'Wymagaj' THEN 2 ELSE (CASE WHEN [RequestMppTransfers] LIKE 'Blokada' THEN 1 ELSE 0 END) END FROM [#tblContractor]),
						'System.Int16, mscorlib, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089', NEWID(), 'splitPayment', 0, 0
					END
					--	idFK
					SET @exAttributeId = NULL
					SELECT @exAttributeId = ex.[ElementExAttrId], @exAttributeGuid = ex.[Guid], @exAttributeCLRType = ex.[OriginalCLRType],
							@exAttributeTitle = ex.[Title], @exAttributeAccessFlag = ex.[AccessFlag], @exAttributeInheritedFlag = ex.[InheritedFlag]
						FROM [SSCommon].[STElementExAttributes] ex
						INNER JOIN [SSCommon].[STElements] elem ON elem.[ElementId] = ex.[ElementId]
						INNER JOIN [SSCommon].[STContractors] ctrs ON ctrs.[MainElement] = elem.[Guid]
						WHERE ctrs.[Id] = @contractorId AND ex.[Name] LIKE 'idFK'
					IF (@exAttributeId IS NOT NULL)
					BEGIN
						INSERT INTO #ExtAttributes([ElementId], [Name], [StringValue], 
							[OriginalCLRType], [Guid], [Title], [InheritedFlag], [AccessFlag])
						SELECT @exAttributeId, 'idFK', (SELECT CASE WHEN ISNULL([IdFk], 0) = 0 THEN '' ELSE CAST([IdFk] AS NVARCHAR(1024)) END FROM [#tblContractor]),
							@exAttributeCLRType, @exAttributeGuid, @exAttributeTitle, @exAttributeInheritedFlag, @exAttributeAccessFlag
					END
					ELSE
					BEGIN
						INSERT INTO #ExtAttributes([ElementId], [Name], [StringValue],
							[OriginalCLRType], [Guid], [Title], [InheritedFlag], [AccessFlag])
						SELECT @extAttElementId, 'idFK', (SELECT CASE WHEN ISNULL([IdFk], 0) = 0 THEN '' ELSE CAST([IdFk] AS NVARCHAR(1024)) END FROM [#tblContractor]),
							'System.String, mscorlib, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089', NEWID(), 'idFK', 0, 0
					END
					--	parametrFK
					SET @exAttributeId = NULL
					SELECT @exAttributeId = ex.[ElementExAttrId], @exAttributeGuid = ex.[Guid], @exAttributeCLRType = ex.[OriginalCLRType],
							@exAttributeTitle = ex.[Title], @exAttributeAccessFlag = ex.[AccessFlag], @exAttributeInheritedFlag = ex.[InheritedFlag]
						FROM [SSCommon].[STElementExAttributes] ex
						INNER JOIN [SSCommon].[STElements] elem ON elem.[ElementId] = ex.[ElementId]
						INNER JOIN [SSCommon].[STContractors] ctrs ON ctrs.[MainElement] = elem.[Guid]
						WHERE ctrs.[Id] = @contractorId AND ex.[Name] LIKE 'parametrFK'
					IF (@exAttributeId IS NOT NULL)
					BEGIN					
						INSERT INTO #ExtAttributes([ElementId], [Name], [StringValue], 
							[OriginalCLRType], [Guid], [Title], [InheritedFlag], [AccessFlag])
						SELECT @exAttributeId, 'parametrFK', (SELECT ISNULL([AccountantParameter], '') FROM [#tblContractor]),
							@exAttributeCLRType, @exAttributeGuid, @exAttributeTitle, @exAttributeInheritedFlag, @exAttributeAccessFlag
					END
					ELSE
					BEGIN
						INSERT INTO #ExtAttributes([ElementId], [Name], [StringValue],
							[OriginalCLRType], [Guid], [Title], [InheritedFlag], [AccessFlag])
						SELECT @extAttElementId, 'parametrFK', (SELECT ISNULL([AccountantParameter], '') FROM [#tblContractor]),
							'System.String, mscorlib, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089', NEWID(), 'parametrFK', 0, 0
					END
					--	typCeny
					SET @exAttributeId = NULL
					SELECT @exAttributeId = ex.[ElementExAttrId], @exAttributeGuid = ex.[Guid], @exAttributeCLRType = ex.[OriginalCLRType],
							@exAttributeTitle = ex.[Title], @exAttributeAccessFlag = ex.[AccessFlag], @exAttributeInheritedFlag = ex.[InheritedFlag]
						FROM [SSCommon].[STElementExAttributes] ex
						INNER JOIN [SSCommon].[STElements] elem ON elem.[ElementId] = ex.[ElementId]
						INNER JOIN [SSCommon].[STContractors] ctrs ON ctrs.[MainElement] = elem.[Guid]
						WHERE ctrs.[Id] = @contractorId AND ex.[Name] LIKE 'typCeny'
					IF (@exAttributeId IS NOT NULL)
					BEGIN						
						INSERT INTO #ExtAttributes([ElementId], [Name], [StringValue], 
							[OriginalCLRType], [Guid], [Title], [InheritedFlag], [AccessFlag])
						SELECT @exAttributeId, 'typCeny', (SELECT ISNULL([PriceTypeId], 0) FROM [#tblContractor]),
							@exAttributeCLRType, @exAttributeGuid, @exAttributeTitle, @exAttributeInheritedFlag, @exAttributeAccessFlag
					END
					ELSE
					BEGIN
						INSERT INTO #ExtAttributes([ElementId], [Name], [StringValue],
							[OriginalCLRType], [Guid], [Title], [InheritedFlag], [AccessFlag])
						SELECT @extAttElementId, 'typCeny', (SELECT ISNULL([PriceTypeId], 0) FROM [#tblContractor]),
							'System.String, mscorlib, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089', NEWID(), 'typCeny', 0, 0
					END
					--	rodzajCeny
					SET @exAttributeId = NULL
					SELECT @exAttributeId = ex.[ElementExAttrId], @exAttributeGuid = ex.[Guid], @exAttributeCLRType = ex.[OriginalCLRType],
							@exAttributeTitle = ex.[Title], @exAttributeAccessFlag = ex.[AccessFlag], @exAttributeInheritedFlag = ex.[InheritedFlag]
						FROM [SSCommon].[STElementExAttributes] ex
						INNER JOIN [SSCommon].[STElements] elem ON elem.[ElementId] = ex.[ElementId]
						INNER JOIN [SSCommon].[STContractors] ctrs ON ctrs.[MainElement] = elem.[Guid]
						WHERE ctrs.[Id] = @contractorId AND ex.[Name] LIKE 'rodzajCeny'
					IF (@exAttributeId IS NOT NULL)
					BEGIN						
						INSERT INTO #ExtAttributes([ElementId], [Name], [StringValue], 
							[OriginalCLRType], [Guid], [Title], [InheritedFlag], [AccessFlag])
						SELECT @exAttributeId, 'rodzajCeny', (SELECT ISNULL([SaleKindId],0) FROM [#tblContractor]),
							@exAttributeCLRType, @exAttributeGuid, @exAttributeTitle, @exAttributeInheritedFlag, @exAttributeAccessFlag
					END
					ELSE
					BEGIN
						INSERT INTO #ExtAttributes([ElementId], [Name], [StringValue],
							[OriginalCLRType], [Guid], [Title], [InheritedFlag], [AccessFlag])
						SELECT @extAttElementId, 'rodzajCeny', (SELECT ISNULL([SaleKindId],0) FROM [#tblContractor]),
							'System.String, mscorlib, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089', NEWID(), 'rodzajCeny', 0, 0
					END
					--	rejestr
					SET @exAttributeId = NULL
					SELECT @exAttributeId = ex.[ElementExAttrId], @exAttributeGuid = ex.[Guid], @exAttributeCLRType = ex.[OriginalCLRType],
							@exAttributeTitle = ex.[Title], @exAttributeAccessFlag = ex.[AccessFlag], @exAttributeInheritedFlag = ex.[InheritedFlag]
						FROM [SSCommon].[STElementExAttributes] ex
						INNER JOIN [SSCommon].[STElements] elem ON elem.[ElementId] = ex.[ElementId]
						INNER JOIN [SSCommon].[STContractors] ctrs ON ctrs.[MainElement] = elem.[Guid]
						WHERE ctrs.[Id] = @contractorId AND ex.[Name] LIKE 'rejestr'
					IF (@exAttributeId IS NOT NULL)
					BEGIN
						INSERT INTO #ExtAttributes([ElementId], [Name], [StringValue], 
							[OriginalCLRType], [Guid], [Title], [InheritedFlag], [AccessFlag])
						SELECT @exAttributeId, 'rejestr', (SELECT ISNULL([PayRegistryId], 0) FROM [#tblContractor]),
							@exAttributeCLRType, @exAttributeGuid, @exAttributeTitle, @exAttributeInheritedFlag, @exAttributeAccessFlag
					END
					ELSE
					BEGIN
						INSERT INTO #ExtAttributes([ElementId], [Name], [StringValue],
							[OriginalCLRType], [Guid], [Title], [InheritedFlag], [AccessFlag])
						SELECT @extAttElementId, 'rejestr', (SELECT ISNULL([PayRegistryId], 0) FROM [#tblContractor]),
							'System.String, mscorlib, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089', NEWID(), 'rejestr', 0, 0
					END
					--	formaPlatnosci
					SET @exAttributeId = NULL
					SELECT @exAttributeId = ex.[ElementExAttrId], @exAttributeGuid = ex.[Guid], @exAttributeCLRType = ex.[OriginalCLRType],
							@exAttributeTitle = ex.[Title], @exAttributeAccessFlag = ex.[AccessFlag], @exAttributeInheritedFlag = ex.[InheritedFlag]
						FROM [SSCommon].[STElementExAttributes] ex
						INNER JOIN [SSCommon].[STElements] elem ON elem.[ElementId] = ex.[ElementId]
						INNER JOIN [SSCommon].[STContractors] ctrs ON ctrs.[MainElement] = elem.[Guid]
						WHERE ctrs.[Id] = @contractorId AND ex.[Name] LIKE 'formaPlatnosci'
					IF (@exAttributeId IS NOT NULL)
					BEGIN
						INSERT INTO #ExtAttributes([ElementId], [Name], [StringValue], 
							[OriginalCLRType], [Guid], [Title], [InheritedFlag], [AccessFlag])
						SELECT @exAttributeId, 'formaPlatnosci', (SELECT ISNULL([PaymentFormId], 0) FROM [#tblContractor]),
							@exAttributeCLRType, @exAttributeGuid, @exAttributeTitle, @exAttributeInheritedFlag, @exAttributeAccessFlag
					END
					ELSE
					BEGIN
						INSERT INTO #ExtAttributes([ElementId], [Name], [StringValue],
							[OriginalCLRType], [Guid], [Title], [InheritedFlag], [AccessFlag])
						SELECT @extAttElementId, 'formaPlatnosci', (SELECT ISNULL([PaymentFormId], 0) FROM [#tblContractor]),
							'System.String, mscorlib, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089', NEWID(), 'formaPlatnosci', 0, 0
					END

					SET @extAttExtAttributes = (SELECT * FROM #ExtAttributes FOR XML PATH('ElementExAttributes'), root ('DocumentElement'))
					
					EXEC [SSCommon].[sp_SaveElementAndAttrs] 
						@extAttElementId , 
						@contractorMainElement, 
						@extAttTitle,
						@extAttActive,
						@extAttAccountNo,
						@extAttDescription,
						@contractorElementKindId,
						@extAttShortcut,
						@extAttPosition,
						@extAttExtObjVer,
						@extAttExtAttributes,
						@extAttExtElementKindGuid,
						@extAttExtWriteAction,
						@extAttExtOtherThenOriginal,
						@contextInfo
				END TRY
				BEGIN CATCH
					THROW 50000, N'Nie powioda si aktualizacja kontrahenta', 1
				END CATCH
			END
		END
		
		IF EXISTS (SELECT * FROM [#tblTradeData])
		BEGIN
			DECLARE @discountPercent FLOAT
			SELECT @discountPercent = [DiscountPercent] FROM [#tblTradeData]
			IF (@discountPercent IS NOT NULL AND @discountPercent <> 0)
			BEGIN
				IF EXISTS (SELECT * FROM [HM].[ContractorDiscounts] WHERE [ContractorId] = @contractorId)
				BEGIN
					UPDATE [HM].[CN] 
					SET [cena] = @discountPercent
					WHERE [typ] = 15 and [idpd] = @contractorId
				END
				ELSE
				BEGIN
					INSERT INTO [HM].[CN]([idpd], [cena], [typ], [flag], [subtyp], [idpm], [typpd], [idck], [typpar], [parametr], [typceny], [waluta], [aktywna], [flagcen], [datastart], [dataend], [createdBy])
					VALUES (@contractorId, @discountPercent, 15, 0, 0, NULL, 0, NULL, 0, 0, 0, '%', 0, 0, NULL, NULL, NULL)
				END
			END
			ELSE
			BEGIN
				DELETE FROM [HM].[CN]
				WHERE [idpd] = @contractorId AND [typ] = 15
			END
		END

		DECLARE @markerId INT		
		DECLARE @contractorKindId INT
		DECLARE @catalogId NVARCHAR(100)
		SELECT @catalogId = [CatalogId], @contractorKindId = ISNULL([ContractorKindId], 0), @markerId = ISNULL([MarkerId], 0)
			FROM [#tblContractor]

		DECLARE @contractorKindGuid UNIQUEIDENTIFIER
		IF (@contractorKindId <> 0)
			SELECT @contractorKindGuid = [Guid] FROM [SSCommon].[STElements] WHERE [ElementId] = @contractorKindId
		
		DECLARE @catalogGuid UNIQUEIDENTIFIER
		IF (@catalogId IS NOT NULL AND @catalogId NOT LIKE '')
			SELECT @catalogGuid = [Guid] FROM [SSCommon].[STElements] WHERE [Shortcut] = @catalogId

		DECLARE @markerGuid UNIQUEIDENTIFIER
		IF (@markerId IS NOT NULL AND @markerId <> 0)
			SELECT @markerGuid = [Guid] FROM [SSCommon].[STElements] WHERE [ElementId] = @markerId

		DECLARE @contractorGuid UNIQUEIDENTIFIER
		SELECT @contractorGuid = [Guid] FROM [SSCommon].[STContractors] WHERE [Id] = @contractorId

		IF EXISTS (SELECT * FROM [#tblCustomFields])
		BEGIN
			--SELECT * FROM [#tblCustomFields]
			DECLARE @valueStatement NVARCHAR(MAX)
			DECLARE @newLine NVARCHAR(10)
			SET @newLine = char(13)+char(10)

			DECLARE @setStatement NVARCHAR(MAX)
			SELECT @setStatement = COALESCE(@setStatement + ', ', '') +
					'[' + [ColumnName] + '] = ' + 
					CASE WHEN [Value] IS NULL THEN 'NULL' 
					ELSE (CASE WHEN ([Type] = 'System.Boolean, mscorlib') THEN (CASE WHEN [Value] = 'True' THEN '1' ELSE '0' END)
						ELSE (CASE WHEN ([Type] = 'System.DateTime, mscorlib') THEN 'CAST(''' + [Value] + '''AS datetime2)'
							ELSE (CASE WHEN ([Type] = 'System.String, mscorlib') THEN '''' + [Value] + ''''
								ELSE [Value] END) END) END) END
			FROM [#tblCustomFields]
			--SELECT @setStatement

			DECLARE @insertColumnsStatement NVARCHAR(MAX)
			DECLARE @insertValuesStatement NVARCHAR(MAX)
			SELECT @insertColumnsStatement = COALESCE(@insertColumnsStatement + ', ', '') + '[' + [ColumnName] + ']',
				@insertValuesStatement = COALESCE(@insertValuesStatement + ', ', '') +
					CASE WHEN [Value] IS NULL THEN 'NULL' 
					ELSE (CASE WHEN ([Type] = 'System.Boolean, mscorlib') THEN (CASE WHEN [Value] = 'True' THEN '1' ELSE '0' END)
						ELSE (CASE WHEN ([Type] = 'System.DateTime, mscorlib') THEN 'CAST(''' + [Value] + '''AS datetime2)'
							ELSE (CASE WHEN ([Type] = 'System.String, mscorlib') THEN '''' + [Value] + ''''
								ELSE [Value] END) END) END) END
			FROM [#tblCustomFields]
			--SELECT @insertColumnsStatement
			--SELECT @insertValuesStatement
			
			SET @valueStatement = CONCAT('IF EXISTS(SELECT * FROM [SSCommon].[',@classificationTableName,'] WHERE [ElementId] = ',@contractorId,')',
				@newLine,'BEGIN',
					@newLine,'UPDATE [SSCommon].[',@classificationTableName,']',
					@newLine,'SET ',
					@newLine,@setStatement,
					@newLine,'WHERE [ElementId] = ',@contractorId,
				@newLine,'END',
				@newLine,'ELSE',
				@newLine,'BEGIN',
					@newLine,'INSERT INTO [SSCommon].[',@classificationTableName,']([ElementId], [Guid],',@insertColumnsStatement,')',
					@newLine,'VALUES(',@contractorId,',''',NEWID(),''',',@insertValuesStatement,')',
				@newLine,'END')

			--SELECT @valueStatement
			EXEC (@valueStatement)
		END
		ELSE
		BEGIN
			DECLARE @deleteStatement NVARCHAR(MAX)

			SET @deleteStatement = CONCAT('DELETE FROM [SSCommon].[', @classificationTableName,'] WHERE [ElementId] = ''', @contractorId, '''')
			--SELECT @deleteStatement
			EXEC (@deleteStatement)
		END

		IF NOT EXISTS(SELECT * FROM [SSCommon].[STContractorsClassification] WHERE [ElementId] LIKE @contractorGuid)
		BEGIN
			INSERT INTO [SSCommon].[STContractorsClassification]([Guid],[ElementId],[CDim_ZnacznikKh],[CDim_RodzajKh],[CDim_KatalogKh])
			VALUES (NEWID(), @contractorGuid, @markerGuid, @contractorKindGuid, @catalogGuid)
		END
		ELSE
		BEGIN
			UPDATE [SSCommon].[STContractorsClassification]
			SET [CDim_ZnacznikKh] = @markerGuid, [CDim_RodzajKh] = @contractorKindGuid, [CDim_KatalogKh] = @catalogGuid
			WHERE [ElementId] LIKE @contractorGuid
		END

		--czycimy mieci
		IF (OBJECT_ID('tempdb..[#tblContractor]') IS NOT NULL) DROP TABLE [#tblContractor]
		IF (OBJECT_ID('tempdb..[#tblContact]') IS NOT NULL) DROP TABLE [#tblContact]
		IF (OBJECT_ID('tempdb..[#tblMainAddress]') IS NOT NULL) DROP TABLE [#tblMainAddress]
		IF (OBJECT_ID('tempdb..[#tblCorespondenceAddress]') IS NOT NULL) DROP TABLE [#tblCorespondenceAddress]
		IF (OBJECT_ID('tempdb..[#tblTradeData]') IS NOT NULL) DROP TABLE [#tblTradeData]
		IF (OBJECT_ID('tempdb..[#tblBankAccounts]') IS NOT NULL) DROP TABLE [#tblBankAccounts]
		IF (OBJECT_ID('tempdb..[#tblCustomFields]') IS NOT NULL) DROP TABLE [#tblCustomFields]

		exec [Common].[UpdateContractorRIFromXML] @contractorId, @contractorData, @moduleCode , @contextInfo, @contextInfoP

		COMMIT TRANSACTION;

		exec [Common].[ReadContractorAsXML] @contractorId, @moduleCode

	END TRY
	BEGIN CATCH
		IF @@TRANCOUNT > 0
		BEGIN
			ROLLBACK TRANSACTION;

			DECLARE @ErrorMessage NVARCHAR(4000) = ERROR_MESSAGE();
			DECLARE @ErrorSeverity INT = ERROR_SEVERITY();
			DECLARE @ErrorState INT = ERROR_STATE();
			
			RAISERROR (@ErrorMessage, -- Message text.
               @ErrorSeverity, -- Severity.
               @ErrorState -- State.
               );
		END
	END CATCH
END